;; Generated package description from notmuch-indicator.el  -*- no-byte-compile: t -*-
(define-package "notmuch-indicator" "1.3.0.0.20260118.63859" "Display mode line indicator with notmuch-count(1) output" '((emacs "27.1")) :commit "d25ece77e8ead14e6f37a2d8c05f19cb60362164" :authors '(("Protesilaos Stavrou" . "info@protesilaos.com")) :maintainer '("Protesilaos Stavrou" . "info@protesilaos.com") :keywords '("convenience" "mail") :url "https://github.com/protesilaos/notmuch-indicator")
